package oracle.forms.demos;

import oracle.forms.ui.*;
import oracle.forms.handler.IHandler;
import oracle.forms.properties.ID;
import oracle.forms.engine.*;
import java.awt.event.*;
import java.util.StringTokenizer;

/**
 * <H3><FONT COLOR="#002299">Pluggable Java Components for Oracle Forms Server 6<I>i</I></FONT></H3>
 * This class is a Pluggable Java Component for Oracle Forms, that implements
 * a method for only allowing defined keystrokes to be entered into a field.
 * The class also provides some additional information for the Forms programmer
 * in the form of information about what the last keypress delivered to the
 * field was and the Shift state at the time.
 *
 * The class implements the following Custom properties for the programmer to
 * use:
 * <ul><li>KEY_CODE</li>
 *     <li>KEY_CHARACTER</li>
 *     <li>KEY_MODIFIER</li>
 *     <li>FILTER_TYPE</li>
 *     <li>CUSTOM_FILTER</li></ul>
 * The Properties <b>KEY_CODE</b>, <b>KEY_CHARACTER</b> and <b>KEY_MODIFIER</b>
 * are read only.
 *
 * @version 1.0 04/10/2000 created<br>
 *
 * @author Duncan Mills
 */
public class KeyFilter extends VTextField
{

  /**
   * the property registered to return the ASCII code of the last key-press
   */
  public final static ID KEYCODE        = ID.registerProperty("KEY_CODE");
  /**
   * the property registered to return the character of the last key pressed
   * or a control key enclosed in braces e.g. F1 will set this property to {F1}
   */
  public final static ID KEYCHAR        = ID.registerProperty("KEY_CHARACTER");
  /**
   * the property registered to return the modifier in force when the last key
   * was pressed - will return SHIFT, CTRL or ALT or a combination thereof
   */
  public final static ID KEYMODIFIER    = ID.registerProperty("KEY_MODIFIER");
  /**
   * the property registered to return or set the current filter on the field
   */
  public final static ID FILTERTYPE     = ID.registerProperty("FILTER_TYPE");
  /**
   * the property registered to return or set the custom filter on the field
   */
  public final static ID CUSTOMFILTER   = ID.registerProperty("CUSTOM_FILTER");


  public final static int ALPHA_FILTER        = 0;
  public final static int NUMERIC_FILTER      = 1;
  public final static int ALPHANUMERIC_FILTER = 2;
  public final static int CUSTOM_FILTER       = 3;
  public final static int NO_FILTER           = 4;


  private String    SavedKeyChar = null;
  private char      FilterChar = 0;
  private int       SavedKeyCode;
  private String    SavedModifier;
  private int       FilterState;
  private String[]  FilterArrays = new String[4];
  private String[]  FilterNames = new String[5];
  private boolean   CustomFilterSet = false;


  /**
   * Constructor for the KeyFiler PJC
   */

  public KeyFilter() {
    super();
    try  {
      jbInit();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  private void jbInit() throws Exception {
    // Set up the pre-defined filters
    FilterState = NO_FILTER;
    FilterArrays[ALPHA_FILTER]        = new String ("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ ");
    FilterArrays[NUMERIC_FILTER]      = new String ("1234567890.,");
    FilterArrays[ALPHANUMERIC_FILTER] = new String ("abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ 1234567890.,");
    FilterNames[ALPHA_FILTER]         = new String("ALPHA");
    FilterNames[NUMERIC_FILTER]       = new String("NUMERIC");
    FilterNames[ALPHANUMERIC_FILTER]  = new String("ALPHANUMERIC");
    FilterNames[CUSTOM_FILTER]        = new String("CUSTOM");
    FilterNames[NO_FILTER]            = new String("NONE");

    // Hook into the keyboard
    addKeyListener(new KeyPressAdapter());
    }

/**
 * Implimentation of IView interface
 *
 * @param pid property ID
 * @return    value of the property
 * @see IView
 */
  public Object getProperty(ID pid)
  {
    if ( pid == KEYCODE )
    {
      return ""+ SavedKeyCode;
    }
    else if ( pid == KEYCHAR )
    {
      return "" + SavedKeyChar;
    }
    else if ( pid == KEYMODIFIER )
    {
      return SavedModifier;
    }
    if ( pid == FILTERTYPE )
    {
      return FilterNames[FilterState];
    }
    else if (pid == CUSTOMFILTER)
    {
      if (CustomFilterSet)
      {
        return FilterArrays[CUSTOM_FILTER];
      }
      else
      {
        return null;
      }
    }
    else
    {
      return super.getProperty(pid);
    }
  }

/**
 * Implimentation of IView interface
 *
 * @param pid   property to be set
 * @param value new value of the property
 * @return      true (if the property could be set), false otherwise
 * @see IView
 */
   public boolean setProperty(ID pid, Object value)
  {
    if ( pid == FILTERTYPE )
    {
      String PropValue = value.toString();
      FilterState = 4;
      for (int i = 0; i <= 3;i++)
      {
        if (PropValue.toUpperCase().compareTo(FilterNames[i]) == 0)
        {
          FilterState = i;
          break;
        }
      }
      return true;
    }
    else if (pid == CUSTOMFILTER)
    {
      /*
       * Check the hashcode here as a way to check that the false
       * returned by toString is due to the value being null rather
       * than just the string "false" which could possibly be set as
       * a custom filter by the user
       */
      if ((value.toString().equals("false")) && (value.hashCode() == 1237))
      {
        System.out.println("Null custom Filter");
        CustomFilterSet = false;
      }
      else
      {
        FilterArrays[CUSTOM_FILTER] = value.toString();
        CustomFilterSet = true;
      }
      return true;
    }
    else
    {
      return super.setProperty(pid, value);
    }
  }

  /**
    * Private class to intercept the keyboard actions and store the keystrokes
    * Away.  Also impliments the Filter functionality.
    */
  class KeyPressAdapter
  extends KeyAdapter
  {
    /**
      * Have to save away the Key in keyPresses as the key character
      * is exposed here but not in keyTyped
      */
    public void keyPressed(KeyEvent e)
    {
      String ThisToken;
      //System.out.println(e.paramString());
      SavedKeyCode = e.getKeyCode();
      StringTokenizer stMain = new StringTokenizer(e.paramString(),",");
      // Throw away the event name
      stMain.nextToken();
      // Throw away the Key code =we have that already
      stMain.nextToken();
      // Next token should contain the keyChar or maybe an edit key
      ThisToken = stMain.nextToken();
      if (ThisToken.indexOf('=') > 0)
      {
        FilterChar = e.getKeyChar();
        SavedKeyChar = "" + FilterChar;
        if (FilterChar == 0) // If a modifier is pressed on it's own
        {
          SavedKeyChar = "{Modifier}";
        }
      }
      else
      {
        // save the edit name eclosed in braces
        FilterChar = 0;
        SavedKeyChar = "{" + ThisToken + "}";
      }
      // Last Token (if there is one) will be the modifier
      if (stMain.hasMoreTokens())
      {
        StringTokenizer stSub = new StringTokenizer(stMain.nextToken(),"=");
        stSub.nextToken();
        SavedModifier = stSub.nextToken().toUpperCase();
      }
      else
      {
        SavedModifier = null;
      }
    }

    /**
      * KeyTyped is where we do the filtering and replace the
      * Key Stroke if required
      */
    public void keyTyped(KeyEvent e)
    {
      if ((FilterChar != 0) && (filterRequired(FilterState,FilterChar)))
      {
       e.setKeyChar(java.awt.event.KeyEvent.CHAR_UNDEFINED);
      }
    }

    /**
      * Check the Key typed against the Filter in force.  If the
      * Key is not in the string then return true so it gets
      * filtered out.
      * If no filter is in force, or the key is in the filter string
      * then return false.
      */

    private boolean filterRequired(int FilterType, int KeyChar)
    {
      if (FilterType == NO_FILTER)
      {
        return false;
      }
      else
      {
        if ((FilterType == CUSTOM_FILTER) && (!CustomFilterSet))
        {
          return false;
        }
        else if (FilterArrays[FilterType].indexOf(KeyChar) == -1)
        {
          return true;
        }
        else
        {
          return false;
        }
      }
    }
  }
}


